/**
 * Copyright (c) 2011-2012, Qulice.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met: 1) Redistributions of source code must retain the above
 * copyright notice, this list of conditions and the following
 * disclaimer. 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 3) Neither the name of the Qulice.com nor
 * the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.qulice.maven;

import com.qulice.spi.ValidationException;
import com.qulice.spi.Validator;
import com.ymock.util.Logger;
import org.apache.maven.plugin.MojoFailureException;

/**
 * Check the project and find all possible violations.
 *
 * @author Yegor Bugayenko (yegor@qulice.com)
 * @version $Id: CheckMojo.java 474 2012-04-01 21:53:26Z guard $
 * @goal check
 * @phase verify
 * @threadSafe
 */
public final class CheckMojo extends AbstractQuliceMojo {

    /**
     * Provider of validators.
     */
    private transient ValidatorsProvider provider =
        new DefaultValidatorsProvider();

    /**
     * {@inheritDoc}
     */
    @Override
    protected void doExecute() throws MojoFailureException {
        try {
            this.run();
        } catch (ValidationException ex) {
            Logger.info(
                this,
                "Read our quality policy: http://www.qulice.com/quality.html"
            );
            throw new MojoFailureException("Failure", ex);
        }
    }

    /**
     * Run them all.
     * @throws ValidationException If any of them fail
     */
    private void run() throws ValidationException {
        for (Validator validator : this.provider.external()) {
            validator.validate(this.env());
        }
        for (MavenValidator validator : this.provider.internal()) {
            validator.validate(this.env());
        }
    }

    /**
     * Set provider of validators.
     * @param prov The provider
     */
    protected void setValidatorsProvider(final ValidatorsProvider prov) {
        this.provider = prov;
    }

}
